/*
 * Copyright (c) 2006, TopCoder, Inc. All rights reserved.
 */
using System;
using System.Collections;
using NAnt.Core;
using NAnt.NUnit2.Tasks;
using NAnt.Core.Attributes;
using TopCoder.Test.Framework.Servers;

namespace TopCoder.Test.Framework
{
    /// <summary>
    /// Defines a customized NAnt task which extends from the <c>NUnit2Task</c>. It provides the functionality to start
    /// and initialize the server before testing, and to deinitialize and stop the server after testing, even if there
    /// is any error.
    /// </summary>
    /// <remarks>
    /// The task uses "serverapptest" tag name in the build file. It does not introduce any new attribute to "nunit2"
    /// task. However, it may contain one or more "server" elements indicating the servers need to be set up/cleaned up
    /// before and after the tests. There must be at lease one "server" element nested under the task element in the
    /// build file. For example:<br/>
    /// <![CDATA[<serverapptest printsummary="yes">
    ///     <server class="serverclass1"><!-- server options --></server>
    ///     <server class="serverclass2"><!-- server options --></server>
    ///     <!-- NUnit options -->
    /// </serverapptest>]]><br/>
    /// This class is not thread-safe.
    /// </remarks>
    /// <author>real_vg</author>
    /// <author>TCSDEVELOPER</author>
    /// <version>1.0</version>
    /// <copyright>Copyright (c) 2006, TopCoder, Inc. All rights reserved.</copyright>
    [TaskName("serverapptest")]
    public class ServerAppTestTask : NUnit2Task
    {
        /// <summary>
        /// Represents the status of a server which is not processed yet.
        /// </summary>
        private const byte NONE = 0;

        /// <summary>
        /// Represents the status of a server which is started.
        /// </summary>
        /// <remarks>
        /// Even if the server does not require to be started, the status would be changed.
        /// </remarks>
        private const byte STARTED = 1;

        /// <summary>
        /// Represents the status of a server whose data is initialized.
        /// </summary>
        /// <remarks>
        /// Even if the server does not require to be initialized, the status would be changed.
        /// </remarks>
        private const byte INITIALIZED = 2;

        /// <summary>
        /// Represents the array of all nested elements describing the servers to be set up/cleaned up.
        /// </summary>
        private ServerElement[] servers = null;

        /// <summary>
        /// Gets or sets the array of all nested elements describing the servers to be set up/cleaned up. There must be
        /// at least one server to be set up/cleaned up.
        /// </summary>
        /// <remarks>
        /// The setter should only be used by NAnt core. There is no validation, since NAnt core will guarantee that
        /// the values are valid.
        /// </remarks>
        /// <value>The array of all nested elements describing the server to be set up/cleaned up.</value>
        [BuildElementArray("server", Required = true)]
        public ServerElement[] Servers
        {
            get
            {
                return servers;
            }
            set
            {
                servers = value;
            }
        }

        /// <summary>
        /// Creates a new instance of <c>ServerAppTestTask</c> class.
        /// </summary>
        public ServerAppTestTask()
        {
        }

        /// <summary>
        /// Cleans up the test environment. The servers are deinitialized and then stopped if necessary. The servers are
        /// processed in a reverse order.
        /// </summary>
        /// <remarks>
        /// The servers are deinitialized and stopped according to the status array. When the status of a server is
        /// <c>INITIALIZED</c>, it is deinitialized and stopped. When the status of a server is <c>STARTED</c>, it
        /// is stopped only. When the status of a server is <c>NONE</c>, nothing is done.
        /// </remarks>
        /// <param name="status">The current status of all servers in the task.</param>
        /// <param name="exceptions">The list containing exceptions occurs before the cleaning up.</param>
        /// <exception cref="ServerTaskException">There are errors in or before the cleaning up.</exception>
        private void CleanUp(byte[] status, IList exceptions)
        {
            for (int serverIndex = status.Length - 1; serverIndex >= 0; --serverIndex)
            {
                int currentStatus = status[serverIndex];

                if (currentStatus == NONE)
                {
                    // If the server is not processed (i.e. when starting, the exception occurs), do nothing.
                    // Obtain the server instance should not be done before this, since error may occur again
                    // when getting the server instance.
                    continue;
                }

                // Obtain the server instance.
                // Since the server instance must be obtained already, it is impossible to throw exceptions here.
                Server server = servers[serverIndex].GetServerInstance();

                if (server.InitServer && (currentStatus == INITIALIZED))
                {
                    // When the server is initialized, and the InitData() is called when initializing
                    // Need deinitialize
                    try
                    {
                        server.CleanUpData();
                    }
                    catch (ServerException e)
                    {
                        // When error occurs, save it in the list
                        exceptions.Add(e);
                    }
                }

                if (server.StartServer && (currentStatus != NONE))
                {
                    // When the server is started, and the StartUp() is called when starting
                    // Need shuting down
                    try
                    {
                        server.ShutDown();
                    }
                    catch (ServerException e)
                    {
                        // When error occurs, save it in the list
                        exceptions.Add(e);
                    }
                }
            }

            // When error occurs, throw the exception
            if (exceptions.Count != 0)
            {
                Exception[] causes = new Exception[exceptions.Count];

                exceptions.CopyTo(causes, 0);
                throw new ServerTaskException("The server application testing task fails.", causes);
            }
        }

        /// <summary>
        /// Executes the server application test task. All servers are started/initialized if required before the
        /// execution of the tests. They are also stopped/deinitialized if required after the execution of the tests.
        /// Even if there are errors during the execution, the servers are properly stopped/deinitialized.
        /// </summary>
        /// <remarks>
        /// The stopping/deinitializing is done in a reverse order compared to the starting/initializing.
        /// </remarks>
        /// <exception cref="ServerTaskException">Creating, starting, initializing, deinitialzing or stopping the server
        /// fails; or executing the tests fails.</exception>
        protected override void ExecuteTask()
        {
            // The array to remember the status of the server
            byte[] status = new byte[servers.Length];
            IList exceptions = new ArrayList();

            // Start all servers
            for (int serverIndex = 0; serverIndex < servers.Length; ++serverIndex)
            {
                try
                {
                    Server server = servers[serverIndex].GetServerInstance();

                    if (server.StartServer)
                    {
                        // When the server need to be started, start it
                        server.StartUp();
                    }

                    // If all normal, mark the server has been started, no matter the StartUp() is called or not.
                    status[serverIndex] = STARTED;
                }
                catch (ServerException e)
                {
                    // When error occurs, save it in the list
                    exceptions.Add(e);
                }
            }

            if (exceptions.Count != 0)
            {
                // When error occurs, clean the test environment.
                CleanUp(status, exceptions);
            }

            // Initialize all server data
            for (int serverIndex = 0; serverIndex < servers.Length; ++serverIndex)
            {
                Server server = servers[serverIndex].GetServerInstance();

                try
                {
                    if (server.InitServer)
                    {
                        // When the server need to be initialized, initialize its data
                        server.InitData();
                    }

                    // If all normal, mark the server has been initialized, no matter the InitData() is called or not.
                    status[serverIndex] = INITIALIZED;
                }
                catch (ServerException e)
                {
                    // When error occurs, save it in the list
                    exceptions.Add(e);
                }
            }

            if (exceptions.Count != 0)
            {
                // When error occurs, clean the test environment.
                CleanUp(status, exceptions);
            }

            try
            {
                // Execute the tests
                base.ExecuteTask();
            }
            catch (BuildException e)
            {
                exceptions.Add(e);
            }

            // When tests are executed, no matter error occurs or not, clean up.
            CleanUp(status, exceptions);
        }
    }
}

