/*
 * Copyright (c) 2006, TopCoder, Inc. All rights reserved.
 */
using System;
using NAnt.Core.Types;
using TopCoder.Test.Framework.Servers;

namespace TopCoder.Test.Framework
{
    /// <summary>
    /// Defines helper methods used in this component.
    /// </summary>
    /// <author>TCSDEVELOPER</author>
    /// <version>1.0</version>
    /// <copyright>Copyright (c) 2006, TopCoder, Inc. All rights reserved.</copyright>
    internal sealed class TestingFrameworkHelper
    {
        /// <summary>
        /// Creates a new instance of the <c>TestingFrameworkHelper</c> class.
        /// </summary>
        /// <remarks>
        /// This private constructor prevents the creation of a new instance.
        /// </remarks>
        private TestingFrameworkHelper()
        {
        }

        /// <summary>
        /// Gets the option value of the given NAnt option. The option value is a string.
        /// </summary>
        /// <param name="options">The NAnt option collection where the option is retrieved.</param>
        /// <param name="optionName">The name of the option to be retrieved.</param>
        /// <returns>The value of the NAnt option.</returns>
        /// <exception cref="ServerException"><c>options</c> is <c>null</c>; or the option is missing.</exception>
        public static string GetOptionValue(OptionCollection options, string optionName)
        {
            if (options == null)
            {
                throw new ServerException("The option collection is not set.");
            }

            Option option = options[optionName];

            if (option == null)
            {
                throw new ServerException("The option '" + optionName + "' is missing in the NAnt build file.");
            }

            return option.Value;
        }

        /// <summary>
        /// Gets the option value of the given NAnt option. The option value is a non-negative integer.
        /// </summary>
        /// <param name="options">The NAnt option collection where the option is retrieved.</param>
        /// <param name="optionName">The name of the option to be retrieved.</param>
        /// <param name="maximum">The maximum allowed value.</param>
        /// <returns>The value of the NAnt option.</returns>
        /// <exception cref="ServerException"><c>options</c> is <c>null</c>; or the option is missing; or the option
        /// value is not an integer; or the option value is a negative integer; or the option value is greater than
        /// the maximum allowed value.</exception>
        public static int GetOptionIntValue(OptionCollection options, string optionName, int maximum)
        {
            int optionValue;

            try
            {
                // Parse the integer value from the option value string
                optionValue = int.Parse(GetOptionValue(options, optionName));
            }
            catch (FormatException e)
            {
                throw new ServerException("The option value is not an integer.", e);
            }
            catch (OverflowException e)
            {
                throw new ServerException("The option value is out of integer range.", e);
            }

            if (optionValue < 0)
            {
                // When the value is negative, it is illegal
                throw new ServerException("The option value must be positive integer.");
            }

            if (optionValue > maximum)
            {
                throw new ServerException("The option value cannot be greater than " + maximum + ".");
            }

            return optionValue;
        }
    }
}

