/*
 * Copyright (c) 2006, TopCoder, Inc. All rights reserved.
 */
using System;
using NUnit.Framework;
using TopCoder.Test.Framework.Servers;

namespace TopCoder.Test.Framework
{
    /// <summary>
    /// Tests the functionality of customized exceptions.
    /// </summary>
    /// <author>TCSDEVELOPER</author>
    /// <version>1.0</version>
    /// <copyright>Copyright (c) 2006, TopCoder, Inc. All rights reserved.</copyright>
    [TestFixture]
    public class ExceptionTestCase
    {
        /// <summary>
        /// Represents a detail error message.
        /// </summary>
        private const string DETAIL_MESSAGE = "A detail message";

        /// <summary>
        /// Represents a original cause of the error.
        /// </summary>
        private static readonly Exception CAUSE = new Exception();

        /// <summary>
        /// Tests accuracy of <c>ServerException(string)</c>. The detail error message should be retrieved as set in
        /// the argument.
        /// </summary>
        [Test]
        public void TestServerExceptionStringAccuracy()
        {
            ServerException exception = new ServerException(DETAIL_MESSAGE);

            Assert.AreEqual(DETAIL_MESSAGE, exception.Message, "The detail message should be the same as in argument.");
        }

        /// <summary>
        /// Tests accuracy of <c>ServerException(string, Exception)</c>. The detail error message should be retrieved as
        /// set in the argument. The original cause of the error should be retrieved as set in the argument.
        /// </summary>
        [Test]
        public void TestServerExceptionStringExceptionAccuracy()
        {
            ServerException exception = new ServerException(DETAIL_MESSAGE, CAUSE);

            Assert.AreEqual(DETAIL_MESSAGE, exception.Message, "The detail message should be the same as in argument.");
            Assert.AreEqual(CAUSE, exception.InnerException, "The cause should be the same as in argument.");
        }

        /// <summary>
        /// Tests accuracy of <c>ServerTaskException(string, Exception[])</c> when the array of causes is <c>null</c>.
        /// The detail error message should be retrieved as set in the argument. The cause array should be set as
        /// <c>null</c>. The first cause should be set as <c>null</c>.
        /// </summary>
        [Test]
        public void TestServerTaskExceptionStringExceptionArrayArrayNullAccuracy()
        {
            ServerTaskException exception = new ServerTaskException(DETAIL_MESSAGE, null);

            Assert.AreEqual(DETAIL_MESSAGE, exception.Message, "The detail message should be the same as in argument.");
            Assert.IsNull(exception.InnerExceptions, "The cause array should be null.");
            Assert.IsNull(exception.InnerException, "The first cause should be null.");
        }

        /// <summary>
        /// Tests accuracy of <c>ServerTaskException(string, Exception[])</c> when the array of causes is empty array.
        /// The detail error message should be retrieved as set in the argument. The cause array should be set as
        /// given in the argument. The first cause should be set as <c>null</c>.
        /// </summary>
        [Test]
        public void TestServerTaskExceptionStringExceptionArrayArrayEmptyAccuracy()
        {
            Exception[] causes = new Exception[0];
            ServerTaskException exception = new ServerTaskException(DETAIL_MESSAGE, causes);

            Assert.AreEqual(DETAIL_MESSAGE, exception.Message, "The detail message should be the same as in argument.");
            Assert.AreEqual(causes, exception.InnerExceptions, "The cause array should be set as in argument.");
            Assert.IsNull(exception.InnerException, "The first cause should be null.");
        }

        /// <summary>
        /// Tests accuracy of <c>ServerTaskException(string, Exception[])</c> when the array of causes is empty array.
        /// The detail error message should be retrieved as set in the argument. The cause array should be set as
        /// given in the argument. The first cause should be set as the first cause in the cause array.
        /// </summary>
        [Test]
        public void TestServerTaskExceptionStringExceptionArrayArrayNormalAccuracy()
        {
            Exception[] causes = new Exception[] { CAUSE, new Exception(), new Exception() };
            ServerTaskException exception = new ServerTaskException(DETAIL_MESSAGE, causes);

            Assert.AreEqual(DETAIL_MESSAGE, exception.Message, "The detail message should be the same as in argument.");
            Assert.AreEqual(causes, exception.InnerExceptions, "The cause array should be set as in argument.");
            Assert.AreEqual(CAUSE, exception.InnerException, "The first cause should be set as in argument.");
        }
    }
}

